import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {
  MatTableDataSource
} from "@angular/material/table";
import {LeafGamesystemCalculator} from "../../product-gamesystem-editor/LeafGamesystemCalculator";
import {animate, state, style, transition, trigger} from "@angular/animations";
import {ProductGamesystem} from "../../../../project/game-model/gamesystems/ProductGamesystem";
import {SimpleGamesystem} from "../../../../project/game-model/gamesystems/SimpleGamesystem";
import {ProductState} from "../../../../project/game-model/gamesystems/states/ProductState";
import {State} from "../../../../project/game-model/gamesystems/states/State";

@Component({
  selector: 'app-product-state-editor',
  templateUrl: './product-state-editor.component.html',
  animations: [
    trigger('detailExpand', [
      state('collapsed,void', style({height: '0px', minHeight: '0'})),
      state('expanded', style({height: '*'})),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
  styleUrl: './product-state-editor.component.scss'
})
export class ProductStateEditorComponent implements OnInit{

  @Input() gamesystem: ProductGamesystem | undefined
  @Output('onOpenGamesystemEditor') openGamesystemEditorEmitter = new EventEmitter<SimpleGamesystem>();
  displayedColumns: string[] = [];
  expandedColumns: string[] = []
  datasource = new MatTableDataSource<ProductState>();

  expandedElement: ProductState | null = null;

  ngOnInit() {
    this.gamesystem!.generateFromChildsystems();
    this.generateColumnNamesRecursively(this.gamesystem!, "");
    this.displayedColumns.push('Initial');
    this.expandedColumns = [...this.displayedColumns, 'expand'];
    this.datasource.data = this.gamesystem!.states;
    this.datasource.filterPredicate = (data: ProductState, filter: string) => {
      const leaf_states = LeafGamesystemCalculator.calcLeafStates(data);
      return leaf_states.some((state) => state.stateLabel.toLowerCase().includes(filter))
    }
  }

  generateColumnNamesRecursively(gamesystem: ProductGamesystem, nestedColumnName: string) {
    gamesystem.innerGamesystems.forEach(innerGamesystem => {
      if(innerGamesystem instanceof SimpleGamesystem) {
        this.displayedColumns.push(nestedColumnName + innerGamesystem.componentName);
      } else {
        this.generateColumnNamesRecursively(innerGamesystem as ProductGamesystem, nestedColumnName + innerGamesystem.componentName + ".");
      }
    })
  }


  getLeafState(state: State<any>, i: number) {
    return LeafGamesystemCalculator.calcLeafStates(state)[i];
  }

  clickOnInnerState(leafIndex: number) {
    const leaf_gamesystems = LeafGamesystemCalculator.calcLeafGeamesystems(this.gamesystem!);
    const clicked_gamesystem = leaf_gamesystems[leafIndex];
    this.openGamesystemEditorEmitter.emit(clicked_gamesystem);
  }

  applyFilter(event: KeyboardEvent) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.datasource.filter = filterValue.trim().toLowerCase();
  }

}
